#include "App.h"
#include <cstdlib>
#include <iostream>
#include <string.h>
#include <Roster.h>
#include <syslog.h>
#include <Debug.h>

//#define DEBUG 1

using namespace std;

App::App(int mb)
	: BApplication(APP_SIG), fTeam(-1), fRunner(NULL), fInstances(0), fThreshMB(mb)
{	
	be_roster->StartWatching(BMessenger(this));

	team_id nteam;	
	if( (nteam = be_roster->TeamFor(NETS_SIG)) != B_ERROR )
	{
		fRunner = new BMessageRunner(BMessenger(this), &BMessage(POLL_NETS_MEMORY), POLL_INTERVAL);
		fTeam = nteam;
		fInstances++;
	}
}

App::~App()
{
	if( fRunner != NULL )
		delete fRunner;
	be_roster->StopWatching(BMessenger(this));
}

void
App::MessageReceived(BMessage* msg)
{
	#ifdef DEBUG
		msg->PrintToStream();
	#endif
	
	switch(msg->what)
	{
		case B_SOME_APP_LAUNCHED:
			{
				const char* sig;
				msg->FindString("be:signature", &sig);
				if( sig != NULL  && strcmp(sig, NETS_SIG) == 0 )
				{
					fInstances++;
					if( fTeam == -1 )
					{
						msg->FindInt32("be:team", &fTeam);
						fRunner = new BMessageRunner(BMessenger(this), &BMessage(POLL_NETS_MEMORY), POLL_INTERVAL);
					}
					else
					{
						//this isn't SUPPOSED to happen (net_server is a B_EXCLUSIVE_LAUNCH app), but I've personally seen it.
						cerr<<"net_server Monitor: warning - second instance of net_server launched.\nThis instance won't be tracked, because it probably can't acquire\nthe network device anyway.\n";
						syslog(LOG_WARNING, "net_server Monitor: second instance of net_server launched and disregarded.");
					}
				}
			}
			break;
		case B_SOME_APP_QUIT:
			{
				const char* sig;
				msg->FindString("be:signature", &sig);
				if( sig != NULL && strcmp(sig, NETS_SIG) == 0 )
				{
					#ifdef DEBUG
						if( fRunner == NULL )
							TRESPASS();
					#endif
					fInstances--;
					if( fInstances == 0 )
					{
						delete fRunner;
						fRunner = NULL;
						fTeam = -1;
					}
				}
			}
			break;
		case POLL_NETS_MEMORY:
			if( fTeam > 0 )
			{
				team_info ti;
				get_team_info( fTeam, &ti );
				cout<<"Team: "<<fTeam<<"\nImage count: "<<ti.image_count<<"\nArea count: "<<ti.area_count<<endl;
				
				size_t ram_size = 0;
				int32 cookie =0;
				area_info ainfo;
				while( get_next_area_info( fTeam, &cookie, &ainfo ) == B_OK )
				{
					ram_size += ainfo.ram_size;
				}
				cout<<"RAM area size: "<<ram_size<<endl;
				if( ram_size > MB_SIZE * fThreshMB )
				{
					CycleNetserver();
				}
			}
			break;
		case 'dbg1':
			CycleNetserver();
			break;
		default:
			BApplication::MessageReceived(msg);
	}	
}

void
App::CycleNetserver()
{
/*	thread_info tinfo;
	int32 cookie = 0;
	get_next_thread_info(fTeam, &cookie, &tinfo);
	status_t ktret;
	if( (ktret = kill_team(fTeam) ) != B_OK )
	{
		cout<<strerror(ktret)<<endl;
		exit(1);
	} */
	
	status_t initret;
	BMessenger messenger(NULL, fTeam, &initret);
	if( initret != B_OK )
	{
		cerr<<"Couldn't send message to net_server.\n";
		exit(1);
	}
	messenger.SendMessage(B_QUIT_REQUESTED);

/*	//wait_for_thread implementation - doesn't work
	status_t ret;
	wait_for_thread(tinfo.thread, &ret);*/
	
	team_info teaminfo;
	while( get_team_info(fTeam, &teaminfo) == B_OK )
	{
		snooze(10000);	
	}
	
	system("/boot/beos/system/boot/Netscript");
	team_id nteam;	
	if( (nteam = be_roster->TeamFor(NETS_SIG)) != B_ERROR )
		fTeam = nteam;
	else
		fTeam = -1;
	cout<<"net_server rotated"<<endl;
	syslog(LOG_NOTICE, "net_server Monitor: net_server rotated");
}

int main(int argc, char** argv) 
{	
	int ramthresh;
	if( argc == 1 )
		ramthresh = DEFAULT_RAM_THRESH;
	else if ( strcmp(argv[1], "--help") == 0 )
	{
		cout<<"net_server Monitor: A tiny daemon process to monitor net_server's memory usage.\n\tUSAGE: net_server\\ Monitor [max mb ram]\n";
		exit(2);
	}
	else
		ramthresh = atoi(argv[1]);

	App app(ramthresh);
	app.Run();
	
	return 0;	
}
